/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_cgiirc.c: Spoofs hostnames for CGI:IRC clients
 *
 *  Copyright (C) 2001-2002 Hybrid Development Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_cgiirc.c,v 1.1 2002/08/25 02:18:24 dgl Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"

static void mr_cgiirc(struct Client*, struct Client*, int, char**);

struct Message cgiirc_msgtab = {
  "CGIIRC", 0, 0, 0, 0, MFLG_SLOW | MFLG_UNREG, 0,
  {mr_cgiirc, m_registered, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&cgiirc_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&cgiirc_msgtab);
}

const char *_version = "$Revision: 1.1 $";
#endif

/* This needs to be dynamic */
#define CGIIRC_PASSWORD "test123"

/*
** mr_cgiirc
**      parv[0] = sender prefix
**      parv[1] = password
**      parv[2] = hostname
**      parv[3] = IP address
*/
static void mr_cgiirc(struct Client *client_p,
                  struct Client *source_p,
                  int parc,
                  char *parv[])
{
    if(strncmp(parv[1], CGIIRC_PASSWORD, strlen(CGIIRC_PASSWORD)) == 0)
    {
       /* Make sure that the input is what we're expecting
	* full checking isn't done on the hostname, just enough to check it's
	* not mad */
       if(!BadPtr(parv[2]) && strchr(parv[2], '.') != NULL
	    && !BadPtr(parv[3]) && strchr(parv[3], '.') != NULL
	       && inet_addr(parv[3]) != 0)
	{
	    strlcpy(source_p->host, parv[2], HOSTLEN);

	    strlcpy(source_p->localClient->sockhost, parv[3], HOSTIPLEN);
	    source_p->localClient->ip.sins.sin.s_addr = inet_addr(parv[3]);

	    sendto_one(source_p, "NOTICE AUTH :*** CGI:IRC Host spoofing active");
	}
	else
	    sendto_one(source_p, "NOTICE AUTH :*** Bad CGIIRC command");
    }
}


